package org.elixir_lang.beam.chunk.debug_info.v1.erl_abstract_code.abstract_code_compiler_options.abstract_code.`fun`

import com.ericsson.otp.erlang.OtpErlangList
import com.ericsson.otp.erlang.OtpErlangObject
import com.ericsson.otp.erlang.OtpErlangTuple
import org.elixir_lang.Macro
import org.elixir_lang.beam.chunk.debug_info.v1.erl_abstract_code.abstract_code_compiler_options.AbstractCode
import org.elixir_lang.beam.chunk.debug_info.v1.erl_abstract_code.abstract_code_compiler_options.AbstractCode.ifTag
import org.elixir_lang.beam.chunk.debug_info.v1.erl_abstract_code.abstract_code_compiler_options.abstract_code.Clause
import org.elixir_lang.beam.chunk.debug_info.v1.erl_abstract_code.abstract_code_compiler_options.abstract_code.MacroString
import org.elixir_lang.beam.chunk.debug_info.v1.erl_abstract_code.abstract_code_compiler_options.abstract_code.Scope

object Clauses {
    fun ifToMacroString(term: OtpErlangObject, scope: Scope): MacroString? =
            ifTag(term, TAG) { toMacroString(it, scope) }

    fun toMacroString(term: OtpErlangTuple, scope: Scope): MacroString {
        val clausesMacroString = clausesMacroString(term, scope)
        val string = "fn $clausesMacroString\n" +
                "end"

        return MacroString(string, doBlock = false)
    }

    private const val TAG = "clauses"

    private fun clausesMacroString(term: OtpErlangTuple, scope: Scope): String =
            toClauses(term)
                    ?.let { clausesToMacroString(it, scope) }
                    ?: AbstractCode.missing("clauses", "fun clauses", term)

    private fun clausesToMacroString(clauses: OtpErlangList, scope: Scope): String =
            clauses
                    .joinToString("\n") {
                        Clause.ifToString(it, scope) ?:
                        AbstractCode.unknown("_clause", "fun clause", it)
                    }
                    .let { Macro.adjustNewLines(it, "\n  ") }

    private fun clausesToMacroString(clauses: OtpErlangObject, scope: Scope): String =
            when (clauses) {
                is OtpErlangList -> clausesToMacroString(clauses, scope)
                else -> AbstractCode.unknown("clauses", "fun clauses", clauses)
            }

    private fun toClauses(term: OtpErlangTuple): OtpErlangObject? = term.elementAt(1)
}
